{% extends "base.html" %}
{% block content %}{% spaceless %}
<section>
    <div class="app app-automations">
        <div class="container">
            {% block content_automations %}{% endblock %}
        </div>
    </div>
</section>
{% endspaceless %}{% endblock %}
