{% extends "automations/base.html" %}{% load i18n %}
{% block content_automations %}{% spaceless %}
    <h1>{% trans "Open tasks for" %} {{ request.user.get_full_name }}</h1>
  {% if count == 0 %}
    <div class="alert alert-success">
      {% trans "Currently no tasks for you" %}
    </div>
  {% else %}
    {% include "automations/includes/task_list.html" %}
  {% endif %}
{% endspaceless %}{% endblock %}