{% load crispy_forms_tags i18n %}{% spaceless %}
  <form class="automations form" novalidate action="{{ request.get_full_path }}" method="post">
    {% csrf_token %}
    <div class="crispy">
    {% crispy form %}
    </div>
    {% if request.GET.back %}
      <a href="{{ request.GET.back }}" class="{{ back_classes }}">{% trans "Back" %}</a>
    {% endif %}
    <input type="submit" value="{{ claim|default:_('OK') }}" class="{{ submit_classes }}"/>
  </form>
{% endspaceless %}